@@ -16,6 +16,7 @@ from hyperliquid.utils.signing import (
     OrderRequest,
     OrderType,
     OrderWire,
+    ScheduleCancelAction,
     float_to_usd_int,
     get_timestamp_ms,
     order_request_to_order_wire,
@@ -25,7 +26,7 @@ from hyperliquid.utils.signing import (
     sign_withdraw_from_bridge_action,
     sign_agent,
 )
-from hyperliquid.utils.types import Any, List, Meta, Optional, Tuple, Cloid
+from hyperliquid.utils.types import Any, List, Meta, SpotMeta, Optional, Tuple, Cloid
 
 
 class Exchange(API):
@@ -40,6 +41,7 @@ class Exchange(API):
         meta: Optional[Meta] = None,
         vault_address: Optional[str] = None,
         account_address: Optional[str] = None,
+        spot_meta: Optional[SpotMeta] = None,
     ):
         super().__init__(base_url)
         self.wallet = wallet
@@ -50,8 +52,18 @@ class Exchange(API):
             self.meta = self.info.meta()
         else:
             self.meta = meta
+
+        if spot_meta is None:
+            self.spot_meta = self.info.spot_meta()
+        else:
+            self.spot_meta = spot_meta
+
         self.coin_to_asset = {asset_info["name"]: asset for (asset, asset_info) in enumerate(self.meta["universe"])}
 
+        # spot assets start at 10000
+        for i, spot_pair in enumerate(self.spot_meta["universe"]):
+            self.coin_to_asset[spot_pair["name"]] = i + 10000
+
     def _post_action(self, action, signature, nonce):
         payload = {
             "action": action,
@@ -278,6 +290,33 @@ class Exchange(API):
             timestamp,
         )
 
+    def schedule_cancel(self, time: Optional[int]) -> Any:
+        """Schedules a time (in UTC millis) to cancel all open orders. The time must be at least 5 seconds after the current time.
+        Once the time comes, all open orders will be canceled and a trigger count will be incremented. The max number of triggers
+        per day is 10. This trigger count is reset at 00:00 UTC.
+
+        Args:
+            time (int): if time is not None, then set the cancel time in the future. If None, then unsets any cancel time in the future.
+        """
+        timestamp = get_timestamp_ms()
+        schedule_cancel_action: ScheduleCancelAction = {
+            "type": "scheduleCancel",
+        }
+        if time is not None:
+            schedule_cancel_action["time"] = time
+        signature = sign_l1_action(
+            self.wallet,
+            schedule_cancel_action,
+            self.vault_address,
+            timestamp,
+            self.base_url == MAINNET_API_URL,
+        )
+        return self._post_action(
+            schedule_cancel_action,
+            signature,
+            timestamp,
+        )
+
     def update_leverage(self, leverage: int, coin: str, is_cross: bool = True) -> Any:
         timestamp = get_timestamp_ms()
         asset = self.coin_to_asset[coin]
@@ -342,6 +381,69 @@ class Exchange(API):
             timestamp,
         )
 
+    def create_sub_account(self, name: str) -> Any:
+        timestamp = get_timestamp_ms()
+        create_sub_account_action = {
+            "type": "createSubAccount",
+            "name": name,
+        }
+        signature = sign_l1_action(
+            self.wallet,
+            create_sub_account_action,
+            None,
+            timestamp,
+            self.base_url == MAINNET_API_URL,
+        )
+        return self._post_action(
+            create_sub_account_action,
+            signature,
+            timestamp,
+        )
+
+    def user_spot_transfer(self, usdc: float, to_perp: bool) -> Any:
+        usdc = int(round(usdc, 2) * 1e6)
+        timestamp = get_timestamp_ms()
+        spot_user_action = {
+            "type": "spotUser",
+            "classTransfer": {
+                "usdc": usdc,
+                "toPerp": to_perp,
+            },
+        }
+        signature = sign_l1_action(
+            self.wallet,
+            spot_user_action,
+            self.vault_address,
+            timestamp,
+            self.base_url == MAINNET_API_URL,
+        )
+        return self._post_action(
+            spot_user_action,
+            signature,
+            timestamp,
+        )
+
+    def sub_account_transfer(self, sub_account_user: str, is_deposit: bool, usd: int) -> Any:
+        timestamp = get_timestamp_ms()
+        sub_account_transfer_action = {
+            "type": "subAccountTransfer",
+            "subAccountUser": sub_account_user,
+            "isDeposit": is_deposit,
+            "usd": usd,
+        }
+        signature = sign_l1_action(
+            self.wallet,
+            sub_account_transfer_action,
+            None,
+            timestamp,
+            self.base_url == MAINNET_API_URL,
+        )
+        return self._post_action(
+            sub_account_transfer_action,
+            signature,
+            timestamp,
+        )
+
     def usd_transfer(self, amount: float, destination: str) -> Any:
         timestamp = get_timestamp_ms()
         payload = {
